#ifndef __BACKPORT_DEVCOREDUMP_H
#define __BACKPORT_DEVCOREDUMP_H
#include_next <linux/devcoredump.h>

#if LINUX_VERSION_CODE < KERNEL_VERSION(4,7,0)

#include <linux/scatterlist.h>
#include <linux/slab.h>
/*
 * _devcd_free_sgtable - free all the memory of the given scatterlist table
 * (i.e. both pages and scatterlist instances)
 * NOTE: if two tables allocated and chained using the sg_chain function then
 * this function should be called only once on the first table
 * @table: pointer to sg_table to free
 */
#define _devcd_free_sgtable LINUX_BACKPORT(_devcd_free_sgtable)
static inline void _devcd_free_sgtable(struct scatterlist *table)
{
	int i;
	struct page *page;
	struct scatterlist *iter;
	struct scatterlist *delete_iter;

	/* free pages */
	iter = table;
	for_each_sg(table, iter, sg_nents(table), i) {
		page = sg_page(iter);
		if (page)
			__free_page(page);
	}

	/* then free all chained tables */
	iter = table;
	delete_iter = table;	/* always points on a head of a table */
	while (!sg_is_last(iter)) {
		iter++;
		if (sg_is_chain(iter)) {
			iter = sg_chain_ptr(iter);
			kfree(delete_iter);
			delete_iter = iter;
		}
	}

	/* free the last table */
	kfree(delete_iter);
}

#define dev_coredumpsg LINUX_BACKPORT(dev_coredumpsg)
static inline void dev_coredumpsg(struct device *dev, struct scatterlist *table,
				  size_t datalen, gfp_t gfp)
{
	_devcd_free_sgtable(table);
}
#endif /* < 4.7 */

#endif /* __BACKPORT_DEVCOREDUMP_H */
